"""
Created on 2022-04-11
@description:刘飞
@description:发布子模块单挑数据删除/修改/详情
"""
import sys
import re
from rest_framework.views import APIView
if 'xj_user' in sys.modules:
    from xj_user.services.user_service import UserService
# if 'xj_enroll' in sys.modules:
#     from xj_enroll.service.enroll_services import EnrollServices
from ..services.thread_item_service_v2 import ThreadItemService
from ..utils.custom_response import util_response
from ..utils.request_params_wrapper import request_params_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper
from ..utils.parse_data import parse_data
from ..utils.x_request_wrapper import x_request_wrapper


class ThreadItemAPI(APIView):
    """单挑信息处理，查，改，删"""

    @request_params_wrapper
    def get(self, *args, request_params, **kwargs):
        """
        信息表详情
        修订：
        [2024.12.03] 第二版起，取消ID查找功能，只允许路径拼接UUID查找。
        """
        pk = kwargs.get("pk", None)
        uuid = kwargs.get("uuid", None)
        if not pk and not uuid:
            return util_response(msg="缺少主键uuid 或 id。", err=1000)

        data, error_text = ThreadItemService.detail(pk=pk, uuid=uuid)
        if not error_text:
            return util_response(data=data)
        return util_response(err=47767, msg=error_text)

    def post(self, request):
        # 用户令牌验证
        token = request.META.get('HTTP_AUTHORIZATION', None)
        token_serv, error_text = UserService.check_token(token)
        if error_text:
            return util_response(err=6045, msg=error_text)

        # # 编辑主键验证
        # pk = kwargs.get("pk", None)
        # uuid = kwargs.get("uuid", None)
        # if not pk and not uuid:
        #     return util_response(msg="缺少主键uuid 或 id。", err=1000)

        params = parse_data(request)
        # print("ThreadAdd: token_serv:", token_serv)
        params['user_id'] = token_serv.get('user_id', None)
        # # 表单初步验证
        # validator = ThreadAddValidator(params)
        # is_pass, error = validator.validate()
        # if not is_pass:
        #     return util_response(err=4022, msg=error)
        # 插入服务
        data, err_txt = ThreadItemService.add(params)
        if not err_txt:
            return util_response(data=data)
        return util_response(err=47767, msg=err_txt)

    # @user_authentication_force_wrapper
    @x_request_wrapper
    # def put(self, *args, request, request_params, user_info, **kwargs):
    def put(self, request, request_params, *args, **kwargs):
        """信息表编辑"""
        # print("> ThreadItemAPI::put:", "request:", request, ", request_params:", request_params, ", args:", args, ", kwargs:", kwargs)

        pk = kwargs.get("pk", None)
        uuid = kwargs.get("uuid", None)
        if not pk and not uuid:
            return util_response(msg="缺少主键uuid 或 id。", err=1000)

        data, error_text = ThreadItemService.edit(pk=pk, uuid=uuid, params=request_params)
        if error_text:
            return util_response(err=1002, msg=error_text)

        return util_response(data=data)


    @user_authentication_force_wrapper
    def delete(self, *args, **kwargs):
        pk = kwargs.get("pk", None)
        if not pk:
            return util_response(msg="非法请求", err=1000)

        data, error_text = ThreadItemService.delete(pk)
        if not error_text:
            return util_response()
        return util_response(err=47767, msg=error_text)
